import { EditorView } from '@codemirror/view';
import { Extension } from '@codemirror/state';
import { LanguageSupport } from '@codemirror/language';
import { default as React_2 } from 'react';
import { SQLConfig } from '@codemirror/lang-sql';

declare interface CommonProps {
    /**
     * 编辑器的值
     */
    value?: string;
    /**
     * 编辑器初始化使用的值
     */
    defaultValue?: string;
    /**
     * 编辑器内容发生变化触发的回调函数
     */
    onChange?: (value: unknown) => void;
    /**
     * 针对当前输入添加额外的补全候选
     * @deprecated
     */
    extraCompletions?: string[] | ((word: string) => string[] | Promise<string[]>);
    /**
     * 可以配置的主题
     */
    theme?: "dark" | "light";
    /**
     * 是否可编辑
     */
    editable?: boolean;
    /**
     * 拓展方法
     */
    extensions?: (Extension | LanguageSupport)[];
}

export declare type IEditor = EditorView;

export declare type IFormatOptions = {
    parser<T, O extends unknown>(text: T, configs?: O): T;
};

declare interface JavascriptProps extends CommonProps {
    language: "javascript";
    langOptions?: {
        jsx?: boolean;
        typescript?: boolean;
    };
}

declare interface JsonProps extends CommonProps {
    language: "json";
    langOptions?: never;
}

export declare const ReactCodemirror: React_2.ForwardRefExoticComponent<(ReactCodemirrorProps | StaticCodemirrorProps) & React_2.RefAttributes<{
    format(configs: IFormatOptions): void;
}>>;

declare type ReactCodemirrorProps = JavascriptProps | JsonProps | SqlProps;

declare interface SqlProps extends CommonProps {
    language: "sql";
    langOptions?: SQLConfig;
}

declare interface StaticCodemirrorProps extends CommonProps {
    editable: false;
    onChange: never;
    defaultValue: never;
    language: ReactCodemirrorProps["language"];
    langOptions: ReactCodemirrorProps["langOptions"];
}

export { }
